""" Syllogistic convenience class.

"""

from ..data import Item
from .encoder_syl import SyllogisticEncoder


#: List of syllogistic task identifiers.
SYLLOGISMS = []
for _prem1 in ['A', 'I', 'E', 'O']:
    for _prem2 in ['A', 'I', 'E', 'O']:
        for _fig in ['1', '2', '3', '4']:
            SYLLOGISMS.append(_prem1 + _prem2 + _fig)

#: List of syllogistic responses.
RESPONSES = []
for _quant in ['A', 'I', 'E', 'O']:
    for _direction in ['ac', 'ca']:
        RESPONSES.append(_quant + _direction)
RESPONSES.append('NVC')

#: List of valid syllogisms
VALID_SYLLOGISMS = [
    'AA1', 'AA2', 'AA4', 'AE1', 'AE2', 'AE3', 'AE4', 'AI2', 'AI4', 'AO3', 'AO4', 'EA1', 'EA2',
    'EA3', 'EA4', 'EI1', 'EI2', 'EI3', 'EI4', 'IA1', 'IA4', 'IE1', 'IE2', 'IE3', 'IE4', 'OA3',
    'OA4'
]

#: List of invalid syllogisms
INVALID_SYLLOGISMS = [
    'AA3', 'AI1', 'AI3', 'AO1', 'AO2', 'EE1', 'EE2', 'EE3', 'EE4', 'EO1', 'EO2', 'EO3', 'EO4',
    'IA2', 'IA3', 'II1', 'II2', 'II3', 'II4', 'IO1', 'IO2', 'IO3', 'IO4', 'OA1', 'OA2', 'OE1',
    'OE2', 'OE3', 'OE4', 'OI1', 'OI2', 'OI3', 'OI4', 'OO1', 'OO2', 'OO3', 'OO4'
]

#: Mapping of syllogisms to logically valid (first-order logics) conclusions
SYLLOGISTIC_FOL_RESPONSES = {
    'AA1': ['Aac', 'Iac', 'Ica'],
    'AA2': ['Aca', 'Iac', 'Ica'],
    'AA3': ['NVC'],
    'AA4': ['Iac', 'Ica'],
    'AI1': ['NVC'],
    'AI2': ['Iac', 'Ica'],
    'AI3': ['NVC'],
    'AI4': ['Iac', 'Ica'],
    'AE1': ['Eac', 'Eca', 'Oac', 'Oca'],
    'AE2': ['Oac'],
    'AE3': ['Eac', 'Eca', 'Oac', 'Oca'],
    'AE4': ['Oac'],
    'AO1': ['NVC'],
    'AO2': ['NVC'],
    'AO3': ['Oca'],
    'AO4': ['Oac'],
    'IA1': ['Iac', 'Ica'],
    'IA2': ['NVC'],
    'IA3': ['NVC'],
    'IA4': ['Iac', 'Ica'],
    'II1': ['NVC'],
    'II2': ['NVC'],
    'II3': ['NVC'],
    'II4': ['NVC'],
    'IE1': ['Oac'],
    'IE2': ['Oac'],
    'IE3': ['Oac'],
    'IE4': ['Oac'],
    'IO1': ['NVC'],
    'IO2': ['NVC'],
    'IO3': ['NVC'],
    'IO4': ['NVC'],
    'EA1': ['Oca'],
    'EA2': ['Eac', 'Eca', 'Oac', 'Oca'],
    'EA3': ['Eac', 'Eca', 'Oac', 'Oca'],
    'EA4': ['Oca'],
    'EI1': ['Oca'],
    'EI2': ['Oca'],
    'EI3': ['Oca'],
    'EI4': ['Oca'],
    'EE1': ['NVC'],
    'EE2': ['NVC'],
    'EE3': ['NVC'],
    'EE4': ['NVC'],
    'EO1': ['NVC'],
    'EO2': ['NVC'],
    'EO3': ['NVC'],
    'EO4': ['NVC'],
    'OA1': ['NVC'],
    'OA2': ['NVC'],
    'OA3': ['Oac'],
    'OA4': ['Oca'],
    'OI1': ['NVC'],
    'OI2': ['NVC'],
    'OI3': ['NVC'],
    'OI4': ['NVC'],
    'OE1': ['NVC'],
    'OE2': ['NVC'],
    'OE3': ['NVC'],
    'OE4': ['NVC'],
    'OO1': ['NVC'],
    'OO2': ['NVC'],
    'OO3': ['NVC'],
    'OO4': ['NVC']
}

def encode_task(task):
    """ Encodes a syllogistic task.

    Parameters
    ----------
    task : list(list(str))
        List representation of the syllogism (e.g., [['All', 'A', 'B'], ['Some', 'B', 'C']]).

    Returns
    -------
    str
        Syllogistic task encoding (e.g., 'AI1').

    """

    return SyllogisticEncoder.encode_task(task)

def encode_response(response, task):
    """ Encodes a response to its syllogistic encoding.

    Parameters
    ----------
    response : list(str)
        Syllogistc response in list representation (e.g., ['All', 'A', 'C'])

    task : list(list(str))
        Syllogistic task in list representation (e.g., [['All', 'A', 'B'], ['Some', 'B', 'C']]).

    Returns
    -------
    str
        Syllogistic response encoding (e.g., 'Aac').

    """

    return SyllogisticEncoder.encode_response(response, task)


def decode_response(enc_response, task):
    """ Decodes an encoded syllogistic response by transforming it to the
    corresponding tuple representation and inserting the appropriate terms.

    Parameters
    ----------
    enc_response : str
        Encoded syllogistic response (e.g., 'Aac').

    task : list(str)
        Syllogistic task in the tuple list representation (e.g.,
        [['Some', 'models', 'managers'], ['All', 'models', 'clerks']]).

    Returns
    -------
    list
        List representation of the response to decode.

    """

    if enc_response == 'NVC':
        return [['NVC']]
    if enc_response == ['NVC']:
        return [enc_response]
    if enc_response == [['NVC']]:
        return enc_response

    obj_a = set(task[0][1:]) - set(task[1][1:])
    obj_c = set(task[1][1:]) - set(task[0][1:])

    quant = enc_response[0].replace('A', 'All').replace(
        'I', 'Some').replace('O', 'Some not').replace('E', 'No')
    if enc_response[1:] == 'ac':
        return [[quant, list(obj_a)[0], list(obj_c)[0]]]

    return [[quant, list(obj_c)[0], list(obj_a)[0]]]


class Syllogism():
    """ Syllogistic helper class. Facilitates the extraction of premise
    information as well as encoding and decoding of responses.

    """

    def __init__(self, item):
        """ Constructs the Syllogism based on a given task item.

        Parameters
        ----------
        item : ccobra.Item
            CCOBRA task item container to base this Syllogism helper on.

        """

        #: Instance of the item the Syllogism is constructed on. The instance
        #: is copied in order to prevent reference mismatches from happening.
        self.item = Item(
            item.identifier,
            item.domain,
            item.task_str,
            item.response_type,
            item.choices_str,
            item.sequence_number)

        #: Reference to the task the Syllogism is constructed on.
        self.task = self.item.task

        #: String representation of the task
        self.encoded_task = encode_task(self.task)

        #: List representation of the first premise
        self.p1 = self.task[0]

        #: List representation of the second premise
        self.p2 = self.task[1]

        #: Quantifier of the first premise
        self.quantifier_p1 = self.task[0][0]

        #: Quantifier of the second premise
        self.quantifier_p2 = self.task[1][0]

        #: Figure of the syllogism
        self.figure = int(self.encoded_task[-1])

        # Figure out the figure and identify the terms
        if self.figure == 1:
            self.A, self.B, self.C = self.task[0][1], self.task[0][2], self.task[1][2]
        elif self.figure == 2:
            self.A, self.B, self.C = self.task[0][2], self.task[0][1], self.task[1][1]
        elif self.figure == 3:
            self.A, self.B, self.C = self.task[0][1], self.task[0][2], self.task[1][1]
        elif self.figure == 4:
            self.A, self.B, self.C = self.task[0][2], self.task[0][1], self.task[1][2]

    def encode_response(self, response):
        """ Encodes a given syllogistic response based on the information
        contained in the premises.

        Parameters
        ----------
        response : list(str)
            Syllogistic response in list representation (e.g.,
            ['All', 'clerks', 'managers']).

        Returns
        -------
        str
            String encoding of the response (e.g., 'Aac').

        """

        return encode_response(response, self.item.task)

    def decode_response(self, encoded_response):
        """ Decodes a syllogistic response in string representation based on
        the information stored in the syllogism's premises.

        Parameters
        ----------
        encoded_response : str
            Encoded syllogistic response (e.g., 'Aac').

        Returns
        -------
        list(str)
            List representation of the encoded response (e.g.,
            ['All', 'clerks', 'managers']).

        """

        return decode_response(encoded_response, self.item.task)

    def is_valid_syllogism(self):
        """ Returns true if syllogism is valid, i.e., has a logically valid conclusion.

        Returns
        -------
        bool
            True, if syllogism is valid, i.e., has a logically valid conclusion. False otherwise.

        """

        return self.encoded_task in VALID_SYLLOGISMS

    def logically_valid_conclusions(self):
        """ Returns the list of logically valid (according to first-order logics) conclusions for
        the syllogism.

        Returns
        -------
        list(str)
            List of logically valid conclusions.

        """

        return SYLLOGISTIC_FOL_RESPONSES[self.encoded_task]

    def __str__(self):
        """ Constructs a string representation for the Syllogism object.

        Returns
        -------
        str
            String representation containing the premise, quantifier, figure,
            and term information.

        """

        rep = 'Syllogism:\n'
        rep += '\ttask: {}\n'.format(self.task)
        rep += '\tencoded_task: {}\n'.format(self.encoded_task)
        rep += '\tp1: {}\n'.format(self.p1)
        rep += '\tp2: {}\n'.format(self.p2)
        rep += '\tquantifier_p1: {}\n'.format(self.quantifier_p1)
        rep += '\tquantifier_p2: {}\n'.format(self.quantifier_p2)
        rep += '\tfigure: {}\n'.format(self.figure)
        rep += '\tTerms:\n'
        rep += '\t\tA: {}\n'.format(self.A)
        rep += '\t\tB: {}\n'.format(self.B)
        rep += '\t\tC: {}\n'.format(self.C)
        return rep



""" ORCA Module

.. rubric:: Submodules

.. autosummary::
   :toctree: _autosummary

   ccobra.benchmark
   ccobra.syllogistic

.. rubric:: Classes

.. autoclass:: CCobraData
    :members:
.. autoclass:: CCobraModel
    :members:
.. autoclass:: Item
    :members:

"""


from .data import CCobraData, Item
from .model import CCobraModel
from .domainhandler import CCobraDomainEncoder

from . import syllogistic
from . import benchmark

__version__ = '0.16.0'



""" Handles domain-specific functionality such as encoding/decoding tasks.

"""

class CCobraDomainEncoder():
    """ Domain encoder class interface. Specifies the functions to be implemented by the
    domain-specific encoder instances.

    """

    def encode_task(self, task):
        """ Encodes a task.

        Parameters
        ----------
        task : list(list(str))
            Task in tuple representation.

        Returns
        -------
        str
            Task representation.

        """

        raise NotImplementedError()

    def encode_response(self, response, task):
        """ Encodes a response

        Parameters
        ----------
        response : list(str)
            Response in tuple representation.

        task : list(list(str))
            Task in tuple representation.

        Returns
        -------
        str
            Response representation.

        """

        raise NotImplementedError()



""" CCOBRA benchmarking module.

"""

import json
import os

from . import modelimporter
from ..domainhandler import CCobraDomainEncoder
from . import contextmanager
from ..syllogistic.encoder_syl import SyllogisticEncoder
from ..propositional.encoder_prop import PropositionalEncoder

class ModelInfo():
    """ Model information container. Contains the properties required to initialize and identify
    CCOBRA model instances.

    """

    def __init__(self, model_info, base_path, load_specific_class=None):
        """ Model initialization.

        Parameters
        ----------
        model_info : object
            Benchmark information about the model. Can either be string or dictionary.

        base_path : str
            Base path for handling relative path specifications.

        load_specific_class : str, optional
            Specific class name to load. Is used whenever multiple alternative CCOBRA model classes
            are specified within the model file.

        """

        #: Model filepath
        self.path = None

        #: String for overriding model name with
        self.override_name = None

        #: Class name for dynamic loading. Is used whenever multiple alternative CCOBRA model
        #: classes are specified within the model file.
        self.load_specific_class = load_specific_class

        #: Keyword arguments for the dynamic model instantiation
        self.args = {}

        if isinstance(model_info, str):
            self.path = fix_model_path(model_info, base_path)
        else:
            self.path = fix_model_path(model_info['filename'], base_path)
            self.override_name = model_info.get('override_name', self.override_name)
            self.args = model_info.get('args', self.args)
            self.load_specific_class = model_info.get(
                'load_specific_class', self.load_specific_class)

    def __str__(self):
        return 'path={}, override_name={}, load_specific_class={}, args={}'.format(
            self.path, self.override_name, self.load_specific_class, self.args)

def load_benchmark(benchmark_file):
    """ Loads a benchmark file containing information about the data and models
    to run. Changes relative paths from `benchmark_file` to absolute ones.

    Parameters
    ----------
    benchmark_file : str
        Path to the benchmark file to load.

    Returns
    -------
    list(ModelInfo)
        List of ModelInfo containers.

    """

    benchmark = None

    # Load raw benchmark file content
    with open(benchmark_file) as json_file:
        benchmark = json.load(json_file)

    # Set default type
    if 'type' not in benchmark:
        benchmark['type'] = 'adaption'

    # Fix relative path information
    base_path = os.path.dirname(os.path.abspath(benchmark_file))

    benchmark['data.train'] = fix_rel_path(benchmark.get('data.train'), base_path)
    benchmark['data.train_person'] = fix_rel_path(benchmark.get('data.train_person', ''), base_path)
    benchmark['data.test'] = fix_rel_path(benchmark.get('data.test', ''), base_path)
    benchmark['models'] = [ModelInfo(x, base_path) for x in benchmark['models']]

    if 'domain_encoders' in benchmark:
        encoders = prepare_domain_encoders(benchmark['domain_encoders'])
        benchmark['domain_encoders'] = encoders
    else:
        benchmark['domain_encoders'] = {}

    # Include default encoders if not overridden
    if 'syllogistic' not in benchmark['domain_encoders']:
        benchmark['domain_encoders']['syllogistic'] = SyllogisticEncoder()
    if 'propositional' not in benchmark['domain_encoders']:
        benchmark['domain_encoders']['propositional'] = PropositionalEncoder()

    return benchmark

def prepare_domain_encoders(domain_encoder_paths):
    """ Processes the domain encoder information from the benchmark specification. Handles
    relative paths or path placeholders (e.g., '%ccobra%' mapping to the module directory of
    the local CCOBRA installation).

    Parameters
    ----------
    domain_encoder_paths : dict(str, str)
        Dictionary mapping from domains to encoders.

    Returns
    -------
    dict(str, str)
        Dictionary mapping from domains to encoders with absolute paths.

    """

    domain_encoders = {}
    for domain, domain_encoder_path in domain_encoder_paths.items():
        # Replace internal ccobra path
        if '%ccobra%' in domain_encoder_path:
            package_path = os.path.split(os.path.split(__file__)[0])[0]
            domain_encoder_path = os.path.normpath(
                domain_encoder_path.replace('%ccobra%', package_path))

        # To instantiate the encoder we need to change to its context (i.e., set the PATH variable
        # accordingly).
        enc = None
        with contextmanager.dir_context(domain_encoder_path):
            imp = modelimporter.ModelImporter(domain_encoder_path, superclass=CCobraDomainEncoder)
            enc = imp.instantiate()

        if not enc:
            raise ValueError('Failed to instantiate encoder class.')
        domain_encoders[domain] = enc

    return domain_encoders

def fix_rel_path(path, base_path):
    """ Fixes relative paths by prepending the benchmark filepath.

    Parameters
    ----------
    path : str
        Path to fix.

    base_path : str
        Basepath used to fix relative paths with. Is prepended to the relative path.

    Returns
    -------
    str
        Fixed absolute path.

    """

    if path and not os.path.isabs(path):
        return os.path.normpath(base_path + os.sep + path)
    return path

def fix_model_path(path, base_path=None):
    """ Fixes the model path by checking if the path directly refers to a python file. Otherwise
    searches for a subdirectory containing possible modules.

    Parameters
    ----------
    path : str
        Model path to fix.

    base_path : str, optional
        Base path to fix the model path with if it is relative.

    Returns
    -------
    str
        Path pointing to the file assumed to contain the model.

    """

    abs_path = path
    if base_path:
        abs_path = fix_rel_path(path, base_path)

    if os.path.isfile(abs_path) and abs_path[-2:] == "py":
        return abs_path

    python_files = []
    sub_directories = []
    for f_name in os.listdir(abs_path):
        f_path = os.path.join(abs_path, f_name)

        if os.path.isfile(f_path) and f_name[-2:] == "py":
            python_files.append(f_path)
        elif os.path.isdir(f_path) and f_name[0] != "." and f_name[:2] != "__":
            sub_directories.append(f_path)

    if python_files:
        return abs_path

    if len(sub_directories) == 1:
        python_files = [
            os.path.join(sub_directories[0], f) for f in os.listdir(
                sub_directories[0]) if os.path.isfile(
                    os.path.join(sub_directories[0], f)) and f[-2:] == "py"]
        if python_files:
            return sub_directories[0]

    raise ValueError("Could not identify model to load for '{}'".format(path))



""" Definition of the evaluative metrics used by CCOBRA.

"""

import http.server as httpserver
import webbrowser

def load_in_default_browser(html):
    """ Opens the html content in the default web browser by temporarily launching a http get
    server and providing the CCOBRA html output.

    Parameters
    ----------
    html : str
        Html content to serve.

    """

    class RequestHandler(httpserver.BaseHTTPRequestHandler):
        """ CCOBRA html request handler.

        """

        def do_GET(self):
            """ Implement the get request handler responding with the html content.

            """

            self.protocol_version = 'HTTP/1.0'

            self.send_response(200)
            self.send_header('Content-type', 'text/html')
            self.send_header('Content-length', len(html))
            self.end_headers()

            buffer_size = 1024 ** 2
            for idx in range(0, len(html), buffer_size):
                self.wfile.write(html[idx:idx+buffer_size])

    server = httpserver.HTTPServer(('127.0.0.1', 0), RequestHandler)
    webbrowser.open('http://127.0.0.1:{}'.format(server.server_port))
    server.handle_request()



""" Model import context manager to temporarily switch working directories.

"""

import sys
import os
from contextlib import contextmanager


@contextmanager
def dir_context(path):
    """ Context manager for the working directory. Stores the current working directory before
    switching it. Finally, resets to the old wd.

    Parameters
    ----------
    path : str
        String to set the working directory to.

    """

    context = os.path.abspath(path)
    if os.path.isfile(context):
        context = os.path.dirname(context)

    old_dir = os.getcwd()
    os.chdir(context)
    sys.path.append(context)
    try:
        yield
    finally:
        os.chdir(old_dir)
        sys.path.remove(context)



""" Contains different comparator classes for model output data structures.

"""

import copy

def tuple_to_string(tuptup):
    """ Converts a tuple to its string representation. Uses different separators (;, /, |) for
    different depths of the representation.

    Parameters
    ----------
    tuptup : list
        Tuple to convert to its string representation.

    Returns
    -------
    str
        String representation of the input tuple.

    """

    def join_deepest(tup, sep=';'):
        """ Recursive function to create the string representation for the deepest level of the
        tuptup list.

        Parameters
        ----------
        tup : object
            Element to join if list or list of lists.

        sep : str, optional
            Separation character to join the list elements by.

        Returns
        -------
        object
            List containing joined string in max depth. Str if input depth = 1.

        """

        if not isinstance(tup, list):
            return tup
        if not isinstance(tup[0], list):
            return sep.join(tup)

        for idx, val in enumerate(tup):
            tup[idx] = join_deepest(val, sep)
        return tup

    tup = copy.deepcopy(tuptup)
    tup = join_deepest(tup, ';')
    tup = join_deepest(tup, '/')
    tup = join_deepest(tup, '|')
    return tup

class Comparator():
    """ Comparator base class.

    """

    def compare(self, obj_a, obj_b):
        """ Base comparison method.

        Parameters
        ----------
        obj_a : object
            Object A for comparison.

        obj_b : object
            Object B for comparison.

        Returns
        -------
        object
            Comparison result.

        """

        raise NotImplementedError()

class EqualityComparator():
    """ Equality comparator. Checks if both responses are equal.

    """

    @staticmethod
    def compare(obj_a, obj_b):
        """ Compares two response objects based on equality.

        Parameters
        ----------
        obj_a : tuple
            Response tuple A for comparison.

        obj_b : tuple
            Response tuple B for comparison.

        Returns
        -------
        bool
            True if both objects are equal, false otherwise.

        """

        return tuple_to_string(obj_a) == tuple_to_string(obj_b)

class NVCComparator():
    """ NVC response comparator. Performs the evaluation based on NVC and non-NVC classes.

    """

    @staticmethod
    def compare(obj_a, obj_b):
        """ Compares two response objects based on their NVCness. Only returns true if both
        responses are in agreement with either responding NVC or not NVC.

        Parameters
        ----------
        obj_a : tuple
            Response tuple A for comparison.

        obj_b : tuple
            Response tuple B for comparison.

        Returns
        -------
        bool
            True only if both objects agree on whether the response is NVC or not.

        """

        return (tuple_to_string(obj_a) == 'NVC') == (tuple_to_string(obj_b) == 'NVC')



""" Imports a given python script and instantiates the contained model if
available.

Copyright 2018 Cognitive Computation Lab
University of Freiburg
Nicolas Riesterer <riestern@tf.uni-freiburg.de>
Daniel Brand <daniel.brand@cognition.uni-freiburg.de>

"""

import argparse
import os
import sys
from contextlib import contextmanager

import pandas as pd

from . import evaluator
from . import server
from . import comparator
from . import benchmark as bmark
from .visualization import html_creator, viz_plot

def parse_arguments():
    """ Parses the command line arguments for the benchmark runner.

    Returns
    -------
    dict
        Dictionary mapping from cmd arguments to values.

    """

    parser = argparse.ArgumentParser(description='')
    parser.add_argument('benchmark', type=str, help='Benchmark file.')
    parser.add_argument('-m', '--model', type=str, help='Model file.')
    parser.add_argument(
        '-o', '--output', type=str, default='browser', help='Output style (browser/html).')
    parser.add_argument('-c', '--cache', type=str, help='Load specified cache file.')
    parser.add_argument('-s', '--save', type=str, help='Store results as csv table.')
    parser.add_argument(
        '-cn', '--classname', type=str,
        help='Load a specific class from a folder containing multiple classes.')

    args = vars(parser.parse_args())

    if not args['model'] and not args['benchmark']:
        print('ERROR: Must specify either model or benchmark.')
        parser.print_help()
        sys.exit(99)

    return args

@contextmanager
def silence_stdout(silent, target=os.devnull):
    """ Contextmanager to silence stdout printing.

    Parameters
    ----------
    silent : bool
        Flag to indicate whether contextmanager should actually silence stdout.

    target : filepath, optional
        Target to redirect silenced stdout output to. Default is os.devnull.
        Can be modified to point to a log file instead.

    """

    new_target = open(target, 'w') if silent else sys.stdout
    old_target, sys.stdout = sys.stdout, new_target
    try:
        yield new_target
    finally:
        sys.stdout = old_target

def main(args):
    """ Main benchmark routine. Parses the arguments, loads models and data,
    runs the evaluation loop and produces the output.

    Parameters
    ----------
    args : dict
        Command line argument dictionary.

    """

    # Compose the model list
    modellist = []
    if args['model']:
        modellist.append(bmark.ModelInfo(args['model'], os.getcwd(), args['classname']))

    # Load the benchmark settings
    benchmark = None
    benchmark = bmark.load_benchmark(args['benchmark'])
    corresponding_data = False
    if 'corresponding_data' in benchmark:
        corresponding_data = benchmark['corresponding_data']

    # Only extend if not cached
    cache_df = None
    if not args['cache']:
        modellist.extend(benchmark['models'])
    else:
        cache_df = pd.read_csv(args['cache'])

    # Extract comparator settings from benchmark description
    eval_comparator = comparator.EqualityComparator()
    if 'comparator' in benchmark:
        if benchmark['comparator'] == 'nvc':
            eval_comparator = comparator.NVCComparator()

    # Run the model evaluation
    is_silent = (args['output'] in ['html', 'server'])
    eva = None
    if benchmark['type'] == 'adaption':
        eva = evaluator.AdaptionEvaluator(
            modellist,
            eval_comparator,
            benchmark['data.test'],
            train_datafile=benchmark['data.train'],
            train_data_person=benchmark['data.train_person'],
            silent=is_silent,
            corresponding_data=corresponding_data,
            domain_encoders=benchmark['domain_encoders'],
            cache_df=cache_df
        )
    elif benchmark['type'] == 'coverage':
        # Check for benchmark validity
        if benchmark['data.train'] or benchmark['data.train_person']:
            print('WARNING: Ignoring specified training and train_person data ' \
                  + 'for coverage evaluation...')

        eva = evaluator.CoverageEvaluator(
            modellist,
            eval_comparator,
            benchmark['data.test'],
            train_datafile=benchmark['data.train'],
            train_data_person=benchmark['data.train_person'],
            silent=is_silent,
            corresponding_data=corresponding_data,
            domain_encoders=benchmark['domain_encoders'],
            cache_df=cache_df
        )
    else:
        raise ValueError('Unknown benchmark type: {}'.format(benchmark['type']))

    with silence_stdout(is_silent):
        res_df = eva.evaluate()

    if 'save' in args:
        res_df.to_csv(args['save'], index=False)

    # Run the metric visualizer
    htmlcrtr = html_creator.HTMLCreator([
        viz_plot.AccuracyVisualizer(),
        viz_plot.BoxplotVisualizer(),
        viz_plot.TableVisualizer()
    ])

    # Prepare the benchmark output information and visualize the evaluation results
    benchmark_info = {
        'name': os.path.basename(args['benchmark']),
        'data.train': os.path.basename(
            benchmark['data.train']) if benchmark['data.train'] else '',
        'data.train_person': os.path.basename(
            benchmark['data.train_person']) if benchmark['data.train_person'] else '',
        'data.test': os.path.basename(benchmark['data.test']),
        'type': benchmark['type'],
        'corresponding_data': benchmark['corresponding_data'],
        'domains': list(res_df['domain'].unique()),
        'response_types': list(res_df['response_type'].unique()),
    }

    if args['output'] == 'browser':
        html = htmlcrtr.to_html(res_df, benchmark_info, embedded=False)
        server.load_in_default_browser(html.encode('utf8'))
    elif args['output'] == 'server':
        html = htmlcrtr.to_html(res_df, benchmark_info, embedded=True)
        sys.stdout.buffer.write(html.encode('utf-8'))
    elif args['output'] == 'html':
        html = htmlcrtr.to_html(res_df, benchmark_info, embedded=False)
        print(html)

def entry_point():
    """ Entry point for the CCOBRA executables.

    """

    args = parse_arguments()

    try:
        main(args)
    except Exception as exc:
        if args['output'] != 'html':
            raise
        msg = 'Error: ' + str(exc)
        if args['output'] == 'html':
            print('<p>{}</p><script>document.getElementById(\"result\").style.backgroundColor ' \
                '= \"Tomato\";</script>'.format(msg))
        else:
            print(exc)
        sys.exit()

if __name__ == '__main__':
    entry_point()
